@@ -22,7 +22,7 @@ dataset_classnames = {
 
 class ALFA_Utils:
 
-    def __init__(self, alfa_results_dir, alfa_path_detections, alfa_path_gt, save_alfa=True, save_gt=False):
+    def __init__(self, alfa_results_dir=None, alfa_path_detections=None, alfa_path_gt=None, save_alfa=False, save_gt=False):
         """
          ALFA Utilities class
 
@@ -39,16 +39,23 @@ class ALFA_Utils:
             Save and transfer of the ground truth results?
 
         """
-        self.__alfa_results_dir = alfa_results_dir
-        ALFA_Utils.__create_dir(self.__alfa_results_dir)
-        self.__alfa_path_detections = alfa_path_detections
-        ALFA_Utils.__create_dir(self.__alfa_path_detections)
-        self.__alfa_path_ground_truth = alfa_path_gt
-        ALFA_Utils.__create_dir(self.__alfa_path_ground_truth)
+        if alfa_results_dir is not None:
+            self.__alfa_results_dir = alfa_results_dir
+            ALFA_Utils.__create_dir(self.__alfa_results_dir)
+        if alfa_path_detections is not None:
+            self.__alfa_path_detections = alfa_path_detections
+            ALFA_Utils.__create_dir(self.__alfa_path_detections)
+        if alfa_path_gt is not None:
+            self.__alfa_path_ground_truth = alfa_path_gt
+            ALFA_Utils.__create_dir(self.__alfa_path_ground_truth)
 
         self.save_alfa = save_alfa
         self.save_ground_truth = save_gt
 
+        self.ai4adas_image_name = ''
+        self.__ai4adas_image_id = -1
+        self.__ai4adas_objects_dict = {}
+
     def transfer_ground_truth_to_alfa_path(self, image_file_name, ai4adas_object):
         """Save and transfer single ALFA specific formatted object as ground truth json."""
 
@@ -93,6 +100,7 @@ class ALFA_Utils:
             print(
                 f'\nCreating ALFA directory {dir_to_create}...')
 
+    @staticmethod
     def box_corner_to_center(box):
         """Convert from (upper-left, lower-right) to (center, width, height)."""
 
@@ -104,6 +112,7 @@ class ALFA_Utils:
         box = torch.stack((cx, cy, w, h), axis=-1).numpy()
         return box
 
+    @staticmethod
     def box_center_to_corner(box):
         """Convert from (center, width, height) to (upper-left, lower-right)."""
 
@@ -134,6 +143,7 @@ class ALFA_Utils:
             alfa_labels.append(dataset_classnames['PASCAL VOC'].index(
                 'bicycle'))  # ALFA bicycle
 
+    @staticmethod
     def to_ai4adas_label(label, ai4adas_labels):
         """Convert YOLO to AI4ADAS class ids/label."""
 
@@ -164,6 +174,7 @@ class ALFA_Utils:
             if alfa_labels[i] < len(dataset_classnames['PASCAL VOC']):
                 class_scores[i, alfa_labels[i]+1] = cls_scs[i]
 
+    @staticmethod
     def prepare_for_alfa(detections, objects_pred, tail_img_name, for_alfa_detections, forJson=True, objects_labels=None, image_shape=(1, 1, 0)):
         """Save the ALFA related information."""
 
@@ -222,52 +233,83 @@ class ALFA_Utils:
         else:
             return class_name[0].lower() + class_name[1:]
 
-    def detections_to_ai4adas_format(detections, objects_pred, ai4adas_objects, image_shape=(1, 1, 0)):
+    def __prepare_ai4adas_objects_dict(self, image_index_up=True):
+        if image_index_up:
+            self.__ai4adas_image_id += 1
+        self.__ai4adas_objects_dict = {"img-name": self.ai4adas_image_name,
+                                       "img-id": self.__ai4adas_image_id, "objects": []}
+
+    def get_ai4adas_objects_dict(self):
+        return self.__ai4adas_objects_dict
+
+    def get_none_detection_ai4adas_objects_dict(self):
+        self.__prepare_ai4adas_objects_dict()
+        return self.__ai4adas_objects_dict
+
+    @staticmethod
+    def __to_ai4adas_class_score(class_score):
+        return str(round(class_score*100, 2))
+
+    def detections_to_ai4adas_format(self, detections, objects_pred, image_shape=(1, 1, 0)):
         """Save the AI4ADAS related information in a AI4ADAS specific format."""
 
-        id = 0
+        object_id = 0
         class_scores = detections[:, 7].numpy()
         h, w = ALFA_Utils.__check_image_shape(image_shape)
+        self.__prepare_ai4adas_objects_dict()
 
         for obj in objects_pred:
             if obj.type in ['DontCare']:
                 continue
             class_name = ALFA_Utils.__first_lowercase(str(obj.type))
-            ai4adas_objects.append({"obj_id": str(id), "class_name": class_name,
-                                    "score": str(class_scores[id]), "xmin": str(obj.box2d[0]/w), "ymin": str(obj.box2d[1]/h), "xmax": str(obj.box2d[2]/w), "ymax": str(obj.box2d[3]/h)})
-            id += 1
-        # print(ai4adas_objects)
+            self.__ai4adas_objects_dict['objects'].append({"obj_id": str(object_id), "class_name": class_name,
+                                                           "score": ALFA_Utils.__to_ai4adas_class_score(class_scores[object_id]), "xmin": str(obj.box2d[0]/w), "ymin": str(obj.box2d[1]/h), "xmax": str(obj.box2d[2]/w), "ymax": str(obj.box2d[3]/h)})
+            object_id += 1
 
-    def to_ai4adas_format(bboxes, labels, class_scores, ai4adas_objects, image_shape=(1, 1, 0)):
+        # print(self.__ai4adas_objects_dict)
+        return self.__ai4adas_objects_dict
+
+    def to_ai4adas_format(self, bboxes, labels, class_scores, image_shape=(1, 1, 0)):
         """Save the AI4ADAS related information (from YOLO) in a AI4ADAS specific format."""
 
-        id = 0
+        object_id = 0
         h, w = ALFA_Utils.__check_image_shape(image_shape)
+        self.__prepare_ai4adas_objects_dict()
 
         for label in labels:
             class_name = ALFA_Utils.__first_lowercase(str(label))
-            ai4adas_objects.append({"obj_id": str(id), "class_name": class_name,
-                                    "score": str(class_scores[id]), "xmin": str(bboxes[id][0]/w), "ymin": str(bboxes[id][1]/h), "xmax": str(bboxes[id][2]/w), "ymax": str(bboxes[id][3]/h)})
-            id += 1
-        # print(ai4adas_objects)
+            self.__ai4adas_objects_dict['objects'].append({"obj_id": str(object_id), "class_name": class_name,
+                                                           "score": ALFA_Utils.__to_ai4adas_class_score(class_scores[object_id]), "xmin": str(bboxes[object_id][0]/w), "ymin": str(bboxes[object_id][1]/h), "xmax": str(bboxes[object_id][2]/w), "ymax": str(bboxes[object_id][3]/h)})
+            object_id += 1
 
-    def convert_to_ai4adas_format(bboxes, label_ids, class_scores, ai4adas_objects, image_shape=(1, 1, 0)):
+        # print(self.__ai4adas_objects_dict)
+        return self.__ai4adas_objects_dict
+
+    def convert_to_ai4adas_format(self, bboxes, label_ids, class_scores, image_shape=(1, 1, 0)):
         """Convert the related information (from ALFA) in a AI4ADAS specific format."""
 
-        id = 0
+        object_id = 0
         h, w = ALFA_Utils.__check_image_shape(image_shape)
+        self.__prepare_ai4adas_objects_dict()
 
         for label_id in label_ids:
             if label_id < len(dataset_classnames['PASCAL VOC']):
-                ai4adas_objects.append({"obj_id": str(id), "class_name": ALFA_Utils.__convert_label_id_to_ai4adas_label(label_id),
-                                        "score": str(class_scores[id][label_id+1]), "xmin": str(bboxes[id, 0]/w), "ymin": str(bboxes[id, 1]/h), "xmax": str(bboxes[id, 2]/w), "ymax": str(bboxes[id, 3]/h)})
-            id += 1
-        # print(ai4adas_alfa_full_detections)
+                self.__ai4adas_objects_dict['objects'].append({"obj_id": str(object_id), "class_name": ALFA_Utils.__convert_label_id_to_ai4adas_label(label_id),
+                                                               "score": ALFA_Utils.__to_ai4adas_class_score(class_scores[object_id][label_id+1]), "xmin": str(bboxes[object_id, 0]/w), "ymin": str(bboxes[object_id, 1]/h), "xmax": str(bboxes[object_id, 2]/w), "ymax": str(bboxes[object_id, 3]/h)})
+                object_id += 1
 
-    def dictionaries_to_ai4adas_format(classes_dict, scores_dict, bboxes_dict, ai4adas_objects, image_shape=(1, 1, 0)):
+        # print(self.__ai4adas_objects_dict)
+        return self.__ai4adas_objects_dict
+
+    def dictionaries_to_ai4adas_format(self, image_name, image_id, classes_dict, scores_dict, bboxes_dict, image_shape=(1, 1, 0)):
         """Save dictionaries of bounding boxes et al. in AI4ADAS specific format."""
 
+        object_id = 0
+        self.ai4adas_image_name = image_name
+        self.__ai4adas_image_id = image_id
         h, w = ALFA_Utils.__check_image_shape(image_shape)
+        self.__prepare_ai4adas_objects_dict(False)
+
         scores = scores_dict['ALFA']
         classes = classes_dict['ALFA']
         bboxes = bboxes_dict['ALFA']
@@ -276,8 +318,9 @@ class ALFA_Utils:
             cls_id = int(classes[i])
             if cls_id >= 0:
                 score = scores[i][1:][cls_id]
-                ai4adas_objects.append({"obj_id": str(len(ai4adas_objects)), "class_name": str(dataset_classnames['PASCAL VOC'][classes[i]]),
-                                        "score": str(score), "xmin": str(bboxes[i, 0]/w), "ymin": str(bboxes[i, 1]/h), "xmax": str(bboxes[i, 2]/w), "ymax": str(bboxes[i, 3]/h)})
+                self.__ai4adas_objects_dict['objects'].append({"obj_id": str(object_id), "class_name": str(dataset_classnames['PASCAL VOC'][classes[i]]),
+                                                               "score": self.__to_ai4adas_class_score(score), "xmin": str(bboxes[i, 0]/w), "ymin": str(bboxes[i, 1]/h), "xmax": str(bboxes[i, 2]/w), "ymax": str(bboxes[i, 3]/h)})
+                object_id += 1
 
     def __check_image_shape(image_shape):
         """Check for valid image shape."""
@@ -307,6 +350,7 @@ class ALFA_Utils:
         else:
             return 1, 1
 
+    @ staticmethod
     def from_ai4adas_to_alfa_format(ai4adas_alfa_detections, alfa_detections, image_shapes):
         """Convert information from the AI4ADAS into ALFA specific format."""
 
@@ -318,7 +362,7 @@ class ALFA_Utils:
                 alfa_bboxes.append(
                     [float(det_objs["xmin"])*w, float(det_objs["ymin"])*h, float(det_objs["xmax"])*w, float(det_objs["ymax"])*h])
                 ALFA_Utils.__to_alfa_label(det_objs['class_name'], alfa_labels)
-                class_scores.append(float(det_objs['score']))
+                class_scores.append(float(det_objs['score'])/100)
 
             if alfa_labels:
                 # According to the ALFA labels, there has to be 20 class ids of ALFA.
